package my.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import org.infinispan.Cache;
import org.infinispan.container.entries.InternalCacheEntry;
import org.infinispan.container.entries.MortalCacheEntry;
import org.infinispan.loaders.CacheLoader;
import org.infinispan.loaders.CacheLoaderConfig;
import org.infinispan.loaders.CacheLoaderException;
import org.infinispan.loaders.CacheStore;
import org.infinispan.loaders.CacheStoreConfig;
import org.infinispan.loaders.modifications.Modification;
import org.infinispan.marshall.AbstractMarshaller;
import org.infinispan.marshall.StreamingMarshaller;
import org.infinispan.marshall.jboss.GenericJBossMarshaller;
import org.infinispan.server.core.CacheValue;
import org.infinispan.transaction.xa.GlobalTransaction;
import org.infinispan.util.ByteArrayKey;
import org.infinispan.util.logging.Log;
import org.infinispan.util.logging.LogFactory;

public class SimpleCacheLoader implements CacheLoader,CacheStore{

    private static final Log logger = LogFactory.getLog(SimpleCacheLoader.class);
    private static final int LIFESPAN = 60000;
    private Properties storage;
    
    @Override
    public void init(CacheLoaderConfig config, Cache<?, ?> cache, StreamingMarshaller m) throws CacheLoaderException {
        storage = new Properties();
        try {
            
            InputStream urlIS = Thread.currentThread().getContextClassLoader().getResourceAsStream("storage.properties"); 
            if(urlIS != null){
                storage.load(urlIS);    
            }
            else{
                storage.load(new FileInputStream(new File("storage.properties")));
            }
            logger.info("--->storage.size:"+storage.size());
            logger.info("--->init");
        }
        catch (IOException e) {
            e.printStackTrace();
            logger.info("--->init Failed");
        }
    }
    @Override
    public InternalCacheEntry load(Object key) throws CacheLoaderException {
        logger.info("--->load");
        AbstractMarshaller marshaller = new GenericJBossMarshaller();
        try {
            Object marshalledObject = marshaller.objectFromByteBuffer(((ByteArrayKey) key).getData());
            String propertyKey = (String)marshalledObject;
            logger.info("key to load:"+propertyKey);
            if (storage.containsKey(propertyKey)){
                String propertyValue = (String)storage.get(propertyKey);
                logger.info("key found:"+propertyValue);
                byte data[] = marshaller.objectToByteBuffer(propertyValue);
                CacheValue cacheValue = new CacheValue(data, 1);
                return new MortalCacheEntry(key, cacheValue, LIFESPAN);
            }
        }
        catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        catch (InterruptedException e) {
            e.printStackTrace();
        }
        logger.info("key not in storage");
        return null;
    }
    @Override
    public Set<InternalCacheEntry> loadAll() throws CacheLoaderException {
        logger.info("--->loadAll");
        return null;
    }
    @Override
    public Set<InternalCacheEntry> load(int numEntries) throws CacheLoaderException {
        logger.info("--->load");
        return null;
    }
    @Override
    public Set<Object> loadAllKeys(Set<Object> keysToExclude) throws CacheLoaderException {
        logger.info("--->loadAllKeys");
        return null;
    }

    @Override
    public boolean containsKey(Object key) throws CacheLoaderException {
        logger.info("--->containsKey");
        return false;
    }

    @Override
    public void start() throws CacheLoaderException {
        logger.info("--->start");
        
    }
    @Override
    public void stop() throws CacheLoaderException {
        logger.info("--->stop");
    }
    @Override
    public Class<? extends CacheLoaderConfig> getConfigurationClass() {
        logger.info("--->getConfigurationClass");
        return null;
    }
    @Override
    public void store(InternalCacheEntry entry) throws CacheLoaderException {
        logger.info("--->store");
        
    }
    @Override
    public void fromStream(ObjectInput inputStream) throws CacheLoaderException {
        logger.info("--->fromStream");
        
    }
    @Override
    public void toStream(ObjectOutput outputStream) throws CacheLoaderException {
        logger.info("--->toStream");
        
    }
    @Override
    public void clear() throws CacheLoaderException {
        logger.info("--->clear");
        
    }
    @Override
    public boolean remove(Object key) throws CacheLoaderException {
        logger.info("--->remove");
        return false;
    }
    @Override
    public void removeAll(Set<Object> keys) throws CacheLoaderException {
        logger.info("--->removeAll");
        
    }
    @Override
    public void purgeExpired() throws CacheLoaderException {
        logger.info("--->purgeExpired");
        
    }
    @Override
    public void prepare(List<? extends Modification> modifications, GlobalTransaction tx, boolean isOnePhase) throws CacheLoaderException {
        logger.info("--->prepare");
        
    }
    @Override
    public void commit(GlobalTransaction tx) throws CacheLoaderException {
        logger.info("--->commit");
        
    }
    @Override
    public void rollback(GlobalTransaction tx) {
        logger.info("--->rollback");
        
    }
    @Override
    public CacheStoreConfig getCacheStoreConfig() {
        logger.info("--->getCacheStoreConfig");
        return null;
    }

}
